// Copyright (c) 2000, 2001
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: DirDialog.java,v 1.3 2001/02/06 03:08:17 maurice Exp $

import java.awt.*;
import java.io.*;
import java.awt.event.*;

class DirDialog extends Dialog implements ActionListener, ItemListener
{
	private List l;
	private TextField dir;
	private String dirName;

	private final int DIR_WIDTH = 30;
	private final int LIST_LEN = 5;

	DirDialog(Frame f, String title, String baseDir)
	{
		super(f, title, true);
		setLayout(new BorderLayout());
		dir = new TextField(DIR_WIDTH);
		dir.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent ae) 
			{
                		String cmd;
                		cmd = ae.getActionCommand();
				setDir(cmd);
			}
		});
		add("North", makeLabeledPanel("Current Directory", dir));
		l = new List(LIST_LEN);
		l.addItemListener(this);
		add("Center", l);
		Panel p = new Panel();
		p.add(makeButton("Select"));
		p.add(makeButton("Cancel"));
		add("South", p);
		pack();
		setDir(baseDir);
	}

	private Panel makeLabeledPanel(String S, Component C)
	{
		Panel p = new Panel();
		p.setLayout(new BorderLayout());
		Label l = new Label(S, Label.CENTER);
		Panel p1 = new Panel();
		p1.add(l);
		Panel p2 = new Panel();
		p2.add(C);
		p.add("North", p1);
		p.add("Center", p2);
		return(p);
	}

	private void setDir(String baseDir, String file)
	{
		try
		{
			File d = new File(baseDir, file);
			setDir(d.getCanonicalPath());
		}
		catch(Exception e) {}
	}

	private void setDir(String baseDir)
	{
		try
		{
			File d = new File(baseDir);
			dir.setText(d.getCanonicalPath());
			if (!dir.getText().endsWith(File.separator))
			{
				dir.setText(dir.getText() + File.separator);
			}
			setDir2(dir.getText());
		}
		catch(Exception e) {}
	}

	private void setDir2(String baseDir)
	{
		try
		{
			File d = new File(baseDir);
			String names[] = d.list(new DirOnly());
			if (l.getItemCount() > 0)
				l.removeAll();
			// horrid kludge - it turns out that the list
			// box does not generate a state change event
			// under EPOC. This kludge doesn't fix it
			// but it does make it look more normal when
			// someone selects another location.
			l.add(".");
			if (d.getParent() != null)
				l.add("..");
			for (int i = 0; i < names.length; i++)
			{
				l.add(names[i]);
			}
		} 
		catch (Exception e) {}
	}

	String getDirectory()
	{
		return dirName;
	}

	Button makeButton(String S)
	{
		Button b = new Button(S);
		b.setActionCommand(S);
		b.addActionListener( (ActionListener) this);
		return(b);
	}

	public void actionPerformed(ActionEvent ae) 
	{
                String cmd;
                cmd = ae.getActionCommand();
		if (cmd.equals("Select"))
		{
			try 
			{
				File d = new File(dir.getText());
				if (!d.exists())
					dirName = null;
				else if (!d.isDirectory())
					dirName = null;
				else
					dirName = d.toString();
			}
			catch (Exception e)
			{
				dirName = null;
			}
			dispose();
		} 
		else if (cmd.equals("Cancel"))
		{
			dirName = null;
			dispose();
		}
	}

	public void itemStateChanged(ItemEvent e) 
	{
		setDir(dir.getText(), l.getSelectedItem());
	}
}
