// Copyright (c) 2000, 2001
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: MainWindow.java,v 1.6 2001/03/03 20:46:21 maurice Exp $

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;

public class MainWindow extends Frame implements ActionListener
{
	private final static int ZIP = 0;
	private final static int UNZIP = 1;
	private final static String BASE_TITLE = "EZipM";
	private int mode = -1;
	private Panel display;
	private List fileList;
	private TextField zipFile;
	private TextField destDir;
	private TextField baseDir;
	private TextField status;
	private Dimension l;

	private final static int FILE_WIDTH = 25;
	private final static int FILE_HEIGHT = 7;
	private final static int ERROR_WIDTH = 85;

	void forceSize()
	{
		// if it occupies over 2/3 of a screen dimension make it all
		Dimension s = getToolkit().getScreenSize();
		Dimension w = getSize();
		Dimension t = new Dimension(w);
		if (s.height <= 3*w.height/2) 
			t.height = s.height;
		if (s.width <= 3*w.width/2) 
			t.width = s.width;
		if (l.height > t.height) 
			t.height = l.height;
		if (l.width > t.width) 
			t.width = l.width;
		setSize(t);
		l = t;
	}

	MenuItem makeMI(String S, int Sc)
	{
		MenuItem mi = new MenuItem(S);
		mi.setActionCommand(S);
		mi.setShortcut(new MenuShortcut(Sc));
		mi.addActionListener( (ActionListener) this);
		return(mi);
	}

	MenuItem makeMI(String S)
	{
		MenuItem mi = new MenuItem(S);
		mi.setActionCommand(S);
		mi.addActionListener( (ActionListener) this);
		return(mi);
	}

	void menu()
	{
		Menu m = new Menu("File");
                m.addSeparator();
                m.add(makeMI("Close", KeyEvent.VK_E));
                Menu h = new Menu("Help");
                h.add(makeMI("About"));
                Menu f = new Menu("Function");
                f.add(makeMI("Zip"));
                f.add(makeMI("UnZip"));
                MenuBar mb = new MenuBar();
                mb.add(m);
                mb.add(f);
                mb.setHelpMenu(h);
                setMenuBar(mb);
	}

	public MainWindow()
	{
		super(BASE_TITLE);
		l = new Dimension();
		this.addWindowListener( new WindowAdapter ()
		{
			public void windowClosing(WindowEvent e)
			{
				close();
			}
		} );
		menu();
		setZip();
		setUnZip();
		add(display);
		show();
	}

	public void actionPerformed(ActionEvent ae) 
	{
		String cmd;
		cmd = ae.getActionCommand();
		if (cmd.equals("Close"))
			close();
		else if (cmd.equals("About"))
			about();
		else if (cmd.equals("Zip"))
			setZip(); 
		else if (cmd.equals("UnZip"))
			setUnZip(); 
		else if (cmd.equals("Add2List"))
			addToList(); 
		else if (cmd.equals("AddD2List"))
			addDToList(false); 
		else if (cmd.equals("AddD22List"))
			addDToList(true); 
		else if (cmd.equals("SelZip"))
			selZip(); 
		else if (cmd.equals("SelBase"))
			selBase(); 
		else if (cmd.equals("SelDest"))
			selDest(); 
		else if (cmd.equals("DelFrList"))
			delFrList(); 
		else if (cmd.equals("ZipFiles"))
			zipFiles(); 
		else if (cmd.equals("ExtAllFile"))
			extAllFile(); 
		else if (cmd.equals("ExtSelFile"))
			extSelFile(); 
		else if (cmd.equals("OpenZip"))
			openZip(); 
	}

	private void close()
	{
		System.exit(0);
	}

	private void about()
	{
		About a = new About(this);
		a.show();
	}

	private Panel makeLabeledPanel(String S, Component C)
	{
		Panel p = new Panel();
		p.setLayout(new BorderLayout());
		Label l = new Label(S);
		Panel p1 = new Panel();
		p1.add(l);
		Panel p2 = new Panel();
		p2.add(C);
		p.add("North", p1);
		p.add("Center", p2);
		return(p);
	}

	private Panel makeLabeledPanel2(String S, Component C)
	{
		Panel p = new Panel();
		p.setLayout(new BorderLayout());
		Label l = new Label(S);
		Panel p1 = new Panel();
		p1.add(l);
		p.add("North", p1);
		p.add("Center", C);
		return(p);
	}

	Button makeButton(String S, String C)
	{
		Button b = new Button(S);
		b.setActionCommand(C);
		b.addActionListener( (ActionListener) this);
		return(b);
	}

	private void setZip()
	{
		if (mode == ZIP)
			return;
		mode = ZIP;
		if (display != null)
			remove(display);
		display = new Panel();
		display.setLayout(new BorderLayout());
		setTitle(BASE_TITLE + " - Zip");
		Panel p = new Panel();
		fileList = new List(FILE_HEIGHT, true);
		p.add(makeLabeledPanel2("List of Files to be Zipped", fileList));
		display.add("West", p);
		p = new Panel();
		p.setLayout(new GridLayout(0,1));
		zipFile = new TextField(FILE_WIDTH);
		zipFile.setEditable(false);
		p.add(makeLabeledPanel("Zip File", zipFile));
		baseDir = new TextField(FILE_WIDTH);
		p.add(makeLabeledPanel("Base Directory", baseDir));
		display.add("East", p);
		status = new TextField(ERROR_WIDTH);
		status.setEditable(false);
		display.add("South", makeLabeledPanel("Status", status));
		Panel buttons = new Panel();
		buttons.setLayout(new GridLayout(0,1));
		buttons.add(makeButton("Add File to File List", "Add2List"));
		buttons.add(makeButton("Add Directory to File List", "AddD2List"));
		buttons.add(makeButton("Add Directories Recursively", "AddD22List"));
		buttons.add(makeButton("Select Zip File", "SelZip"));
		buttons.add(makeButton("Select Base Directory", "SelBase"));
		buttons.add(makeButton("Delete File from File List", "DelFrList"));
		buttons.add(makeButton("Zip Files", "ZipFiles"));
		display.add("Center", buttons);
		add(display);
		pack();
		forceSize();
	}

	private void setUnZip()
	{
		if (mode == UNZIP)
			return;
		mode = UNZIP;
		if (display != null)
			remove(display);
		display = new Panel();
		display.setLayout(new BorderLayout());
		setTitle(BASE_TITLE + " - UnZip");
		zipFile = new TextField(FILE_WIDTH);
		zipFile.setEditable(false);
		Panel p = new Panel();
		p.setLayout(new GridLayout(0,1));
		p.add(makeLabeledPanel("Zip File", zipFile));
		destDir = new TextField(FILE_WIDTH);
		p.add(makeLabeledPanel("Destination Directory", destDir));
		display.add("East", p);
		fileList = new List(FILE_HEIGHT, true);
		zipFile.setEditable(false);
		p = new Panel();
		p.add(makeLabeledPanel2("List of Files to be Unzipped", fileList));
		display.add("West", p);
		status = new TextField(ERROR_WIDTH);
		status.setEditable(false);
		display.add("South", makeLabeledPanel("Status", status));
		Panel buttons = new Panel();
		buttons.setLayout(new GridLayout(0,1));
		buttons.add(makeButton("Open Zip File", "OpenZip"));
		buttons.add(makeButton("Select Destination Directory", "SelDest"));
		buttons.add(makeButton("Extract Selected Files", "ExtSelFile"));
		buttons.add(makeButton("Extract All Files", "ExtAllFile"));
		display.add("Center", buttons);
		add(display);
		pack();
		forceSize();
	}

	private void addToList()
	{
		baseDirChk();
		status.setText("");
		FileDialog fd = new FileDialog(this, "Select File", FileDialog.LOAD);
		try
		{
			if (!baseDir.getText().equals(""))
				fd.setDirectory(baseDir.getText());
		}
		catch (Exception e) {}
		fd.show();
		if (fd.getFile() == null)
			return;
		try
		{
			File file = new File(fd.getDirectory(), fd.getFile());
			fileList.addItem(file.toString());
		}
		catch(Exception e)
		{
			status.setText("Unable to add: " + e.toString());
		}
	} 

	private void addDToList(boolean recurse)
	{
		status.setText("");
		DirDialog dd = new DirDialog(this, "Select Directory to Add", "/");
		dd.show();
		if (dd.getDirectory() == null)
			return;
		try
		{
			File dir = new File(dd.getDirectory());
			AddDir(dir, recurse);
		}
		catch(Exception e)
		{
			status.setText("Unable to add dir: " + e.toString());
		}
	}

	private void AddDir(File dir, boolean recurse) throws java.io.IOException
	{
		String files[] = dir.list();
		for (int i = 0; i < files.length; i++)
		{
			File f = new File(dir, files[i]);
			if (f.exists() && f.isFile())
				fileList.addItem(f.getCanonicalPath());
			if (f.exists() && f.isDirectory() && recurse)
				AddDir(f, recurse);
		}
	}

	private void baseDirChk()
	{
		if (baseDir.getText().equals(""))
			return;
		if (!baseDir.getText().endsWith(File.separator))
		{
			baseDir.setText(baseDir.getText() + File.separator);
		}
	}

	private void selZip()
	{
		status.setText("");
		FileDialog fd = new FileDialog(this, "Select Zip File", FileDialog.SAVE);
		fd.show();
		if (fd.getFile() == null)
			return;
		try
		{
			File file = new File(fd.getDirectory(), fd.getFile());
			zipFile.setText(file.toString());
		}
		catch(Exception e)
		{
			status.setText("Unable to save: " + e.toString());
		}
	} 

	private void selBase()
	{
		status.setText("");
		DirDialog dd = new DirDialog(this, "Select Base Directory", "/");
		dd.show();
		if (dd.getDirectory() != null)
			baseDir.setText(dd.getDirectory());
	} 

	private void selDest()
	{
		status.setText("");
		DirDialog dd = new DirDialog(this, "Select Destination Directory", "/");
		dd.show();
		if (dd.getDirectory() != null)
			destDir.setText(dd.getDirectory());
	} 

	private void delFrList()
	{
		int i;
		status.setText("");
		String items[] = fileList.getSelectedItems();
		for (i = 0; i < items.length; i++)
			fileList.remove(items[i]);
	} 

	private void zipFiles()
	{
		status.setText("");
		baseDirChk();
		try
		{
			byte[] buffer = new byte[4096];
			int bytesRead;
			ZipOutputStream out = new ZipOutputStream(new FileOutputStream(zipFile.getText()));
			for(int i = 0; i < fileList.getItemCount(); i++)
			{
				File f = new File(fileList.getItem(i));
				FileInputStream in = new FileInputStream(f);
				String dest = f.getPath();
				if (dest.startsWith(baseDir.getText()))
				{
					dest = dest.substring(baseDir.getText().length());
				}
				ZipEntry entry = new ZipEntry(dest);
				out.putNextEntry(entry);
				while ((bytesRead = in.read(buffer)) != -1)
				{
					out.write(buffer, 0, bytesRead);
				}
				in.close();
			}
			out.close();
			status.setText("Done");
		}
		catch(Exception e)
		{
			status.setText("Unable to zip: " + e.toString());
		}
	} 

	private void openZip()
	{
		status.setText("");
		FileDialog fd;
		if (mode == UNZIP)
			fd = new FileDialog(this, "Select Zip File", FileDialog.LOAD);
		else
			fd = new FileDialog(this, "Select Zip File", FileDialog.SAVE);
		fd.show();
		if (fd.getFile() == null)
			return;
		try
		{
			File file = new File(fd.getDirectory(), fd.getFile());
			zipFile.setText(file.toString());
			FileInputStream in = new FileInputStream(file);
			ZipInputStream zin = new ZipInputStream(in);
			ZipEntry ze; 
			while ((ze = zin.getNextEntry()) != null)
			{
				fileList.addItem(ze.getName());
			}
		}
		catch(Exception e)
		{
			status.setText("Unable to open: " + e.toString());
		}
	} 

	private void extAllFile()
	{
		status.setText("");
		try
		{
			ZipInputStream zin = new ZipInputStream(new FileInputStream(zipFile.getText()));
			ZipEntry ze; 
			byte[] buffer = new byte[4096];
			int bytesRead;
			while ((ze = zin.getNextEntry()) != null)
			{
				File f = new File(destDir.getText(), ze.getName());
				if (f.exists())
				{
					YesNo yn = new YesNo(this, "Overwrite?", "Overwrite file " + f.getPath());
					yn.show();
					if (!yn.getResult())
					{
						zin.closeEntry();
						continue;
					}
				}
				new File(f.getParent()).mkdirs();
				FileOutputStream out = new FileOutputStream(f);
				while ((bytesRead = zin.read(buffer)) != -1)
				{
					out.write(buffer, 0, bytesRead);
				}
				out.close();
			}
			zin.close();
			status.setText("Done");
		}
		catch(Exception e)
		{
			status.setText("Unable to open: " + e.toString());
		}
	} 

	private void extSelFile()
	{
		status.setText("");
		try
		{
			ZipFile zf = new ZipFile(zipFile.getText());
			ZipEntry ze; 
			byte[] buffer = new byte[4096];
			int bytesRead;
			String files[] = fileList.getSelectedItems();
			for(int i = 0; i < files.length; i++)
			{
				ze = zf.getEntry(files[i]);
				InputStream in = zf.getInputStream(ze);
				File f = new File(destDir.getText(), files[i]);
				if (f.exists())
				{
					YesNo yn = new YesNo(this, "Overwrite?", "Overwrite file " + f.getPath());
					yn.show();
					if (!yn.getResult())
					{
						in.close();
						continue;
					}
				}
				new File(f.getParent()).mkdirs();
				FileOutputStream out = new FileOutputStream(f);
				while ((bytesRead = in.read(buffer)) != -1)
				{
					out.write(buffer, 0, bytesRead);
				}
				out.close();
			}
			zf.close();
			status.setText("Done");
		}
		catch(Exception e)
		{
			status.setText("Unable to open: " + e.toString());
		}
	} 
}
